@@ -25,10 +25,12 @@ SOFTWARE.
 
 from __future__ import annotations
 
+from aiohttp.typedefs import JSONDecoder
+
 
 __title__ = "asyncnt"
 __author__ = "axemalt"
-__version__ = "1.1.0"
+__version__ = "1.2.0"
 
 
 from typing import Optional, Type, List, Dict
@@ -37,6 +39,7 @@ import cloudscraper
 import asyncio
 import aiohttp
 import json
+import time
 import re
 
 
@@ -336,10 +339,16 @@ class Session(cloudscraper.CloudScraper):
 
     __slots__ = ["_session", "headers"]
 
-    def __init__(self, *, session: Optional[aiohttp.ClientSession] = None) -> None:
+    def __init__(self) -> None:
         super().__init__()
 
-        self._session = session or aiohttp.ClientSession()
+        self.per: float = 1
+        self.rate: int = 10
+        self._lock: asyncio.Lock = asyncio.Lock()
+        self._window: float = 0.0
+        self._tokens: int = self.rate
+        self._session: aiohttp.ClientSession = aiohttp.ClientSession()
+        self._semaphore: asyncio.Semaphore = asyncio.Semaphore(value=self.rate)
 
     def __del__(self) -> None:
         if not self._session.closed:
@@ -358,27 +367,59 @@ class Session(cloudscraper.CloudScraper):
 
         await self._session.close()
 
-    async def _get(self, url: str) -> aiohttp.ClientResponse:
-        response: aiohttp.ClientResponse = await self._session.get(
-            url, headers=self.headers
-        )
+    def _update_rate_limit(self) -> Optional[float]:
+        current = time.time()
+
+        if current > self._window + self.per:
+            self._tokens = self.rate
+            self._window = current
+
+        if self._tokens == 0:
+            return self.per - (current - self._window)
+
+        self._tokens -= 1
+
+    async def get(self, url: str) -> Optional[aiohttp.ClientResponse]:
+        """
+        Gets data from Nitro Type.
+
+        :param url: The url to get data from.
+        :type url: str
+        :raise asyncnt.HTTPException: Getting the data failed.
+        :return: The data from the url.
+        :rtype: Optional[aiohttp.ClientResponse]
+        """
+
+        async with self._lock:
+            wait_for = self._update_rate_limit()
+            if wait_for:
+                await asyncio.sleep(wait_for)
+                self._update_rate_limit()
+        
+        async with self._semaphore:
+            response: aiohttp.ClientResponse = await self._session.get(
+                url, headers=self.headers
+            )
 
         if response.status == 200:
             return response
         else:
             raise HTTPException(response)
 
+
     async def get_racer(self, username: str) -> Optional[Racer]:
         """
         Get a racer with a username.
 
         :param username: The racer's username.
         :type username: str
+        :raise asyncnt.InvalidRacerUsername: The username given is invalid.
         :raise asyncnt.HTTPException: Getting the racer failed.
+        :return: The racer with the given username.
         :rtype: Optional[asyncnt.Racer]
         """
 
-        raw_data: aiohttp.ClientResponse = await self._get(
+        raw_data: aiohttp.ClientResponse = await self.get(
             f"https://nitrotype.com/racer/{username}/"
         )
         text: str = await raw_data.text()
@@ -397,11 +438,13 @@ class Session(cloudscraper.CloudScraper):
 
         :param tag: The team's tag.
         :type tag: str
+        :raise asyncnt.InvalidTeamTag: The tag given is invalid.
         :raise asyncnt.HTTPException: Getting the team failed.
+        :return: The team with the given tag.
         :rtype: Optional[asyncnt.Team]
         """
 
-        raw_data: aiohttp.ClientResponse = await self._get(
+        raw_data: aiohttp.ClientResponse = await self.get(
             f"https://nitrotype.com/api/teams/{tag}/"
         )
         data: Dict = await raw_data.json()
